from .particle import Particle
from .particle_generator import ParticleGenerator
import pygame

class ParticleSystem:
    def __init__(self) -> None:
        self.particles = []
        self.generators = []

    def add_particle(self, x: float, y: float, vx: float, vy: float, dvx: float, dvy: float, angle: float, dangle: float, speed: float, lifespan: int, size: int, red: int, green: int, blue: int, alpha: int, shape: str, gradient: bool = False) -> None:
        self.particles.append(Particle(x, y, vx, vy, dvx, dvy, angle, dangle, speed, lifespan, size, red, green, blue, alpha, shape, gradient))

    def add_generator(self, generator: ParticleGenerator) -> None:
        self.generators.append(generator)

    def get_generator_by_name(self, name: str) -> ParticleGenerator:
        for generator in self.generators:
            if generator.name == name:
                return generator
        return None

    def apply_force_to_all(self, fx: float, fy: float) -> None:
        for particle in self.particles:
            particle.apply_force(fx, fy)

    def update(self, delta_time: float = None) -> None:
        particle_x, particle_y = 0, 0
        for generator in self.generators:
            generator.update(delta_time)
        for particle in self.particles:
            particle.update(particle_x, particle_y, delta_time)
        self.particles = [particle for particle in self.particles if particle.lifespan > 0]

    def draw(self, screen: pygame.Surface) -> None:
        for particle in self.particles:
            particle.draw(screen)


from .particle import Particle
from .particle_system import ParticleSystem
from .particle_generator import ParticleGenerator
try:
    from .particle_tester import ParticleTester
except Exception as e:
    print(e)


import pygame
try:
    import pygame_gui
except:
    raise ImportError("Could not import pygame_gui")
import random
from src.particle_system.particle_system import ParticleSystem

class ParticleTester:
    def __init__(self):
        pygame.init()
        self.screen = pygame.display.set_mode((1280, 720))
        pygame.display.set_caption("Particle System GUI")
        self.clock = pygame.time.Clock()
        self.running = True

        self.particle_system = ParticleSystem()
        self.manager = pygame_gui.UIManager((1280, 720))

        self.create_ui_elements()

    def create_ui_elements(self):
        labels = ["X", "Y", "VX", "VY", "DVX", "DVY", "Angle", "DAngle", "Speed", "Lifespan", "Size", "Red", "Green", "Blue", "Alpha", "Shape"]
        defaults = [400, 300, random.uniform(-1, 1), random.uniform(-1, 1), 0, 0, 0, 0.1, 2, 60, 10, 255, 0, 0, 255, 'circle']
        y = 10
        self.input_boxes = []
        for label, default in zip(labels, defaults):
            pygame_gui.elements.UILabel(relative_rect=pygame.Rect((10, y), (100, 30)), text=label, manager=self.manager)
            self.input_boxes.append(pygame_gui.elements.UITextEntryLine(relative_rect=pygame.Rect((120, y), (200, 30)), manager=self.manager))
            self.input_boxes[-1].set_text(str(default))
            y += 40

        self.add_button = pygame_gui.elements.UIButton(relative_rect=pygame.Rect((10, y), (200, 30)), text="Add Particle", manager=self.manager)

    def add_particle(self):
        values = [box.get_text() for box in self.input_boxes]
        x, y, vx, vy, dvx, dvy, angle, dangle, speed, lifespan, size, red, green, blue, alpha, shape = values
        self.particle_system.add_particle(float(x), float(y), float(vx), float(vy), float(dvx), float(dvy), float(angle), float(dangle), float(speed), int(lifespan), int(size), int(red), int(green), int(blue), int(alpha), shape)

    def run(self):
        while self.running:
            time_delta = self.clock.tick(60) / 1000.0
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False
                if event.type == pygame_gui.UI_BUTTON_PRESSED:
                    if event.ui_element == self.add_button:
                        self.add_particle()
                self.manager.process_events(event)

            self.manager.update(time_delta)
            self.screen.fill((30, 30, 30))
            self.manager.draw_ui(self.screen)

            delta_time = self.clock.tick(120) / 1000.0
            self.particle_system.update(delta_time=delta_time)
            self.particle_system.draw(self.screen)
            pygame.display.flip()

        pygame.quit()

if __name__ == "__main__":
    gui = ParticleTester()
    gui.run()


import pygame

class ParticleGenerator:
    def __init__(self, particle_system, name: str, x: float, y: float, vx: float, vy: float, dvx: float, dvy: float, angle: float, dangle: float, speed: float, lifespan: int, size: int, red: int, green: int, blue: int, alpha: int, shape: str, gradient: bool, rate: float) -> None:
        self.particle_system = particle_system
        self.name = name
        self.x = x
        self.y = y
        self.vx = vx
        self.vy = vy
        self.dvx = dvx
        self.dvy = dvy
        self.angle = angle
        self.dangle = dangle
        self.speed = speed
        self.lifespan = lifespan
        self.size = size
        self.red = red
        self.green = green
        self.blue = blue
        self.alpha = alpha
        self.shape = shape
        self.gradient = gradient
        self.rate = rate
        self.time_since_last_particle = 0.0

    def update(self, delta_time: float) -> None:
        self.time_since_last_particle += delta_time
        while self.time_since_last_particle >= 1.0 / self.rate:
            self.particle_system.add_particle(self.x, self.y, self.vx, self.vy, self.dvx, self.dvy, self.angle, self.dangle, self.speed, self.lifespan, self.size, self.red, self.green, self.blue, self.alpha, self.shape, self.gradient)
            self.time_since_last_particle -= 1.0 / self.rate

    def edit(self, x: float = None, y: float = None, vx: float = None, vy: float = None, dvx: float = None, dvy: float = None, angle: float = None, dangle: float = None, speed: float = None, lifespan: int = None, size: int = None, red: int = None, green: int = None, blue: int = None, alpha: int = None, shape: str = None, gradient: bool = None, rate: float = None) -> None:
        if x is not None:
            self.x = x
        if y is not None:
            self.y = y
        if vx is not None:
            self.vx = vx
        if vy is not None:
            self.vy = vy
        if dvx is not None:
            self.dvx = dvx
        if dvy is not None:
            self.dvy = dvy
        if angle is not None:
            self.angle = angle
        if dangle is not None:
            self.dangle = dangle
        if speed is not None:
            self.speed = speed
        if lifespan is not None:
            self.lifespan = lifespan
        if size is not None:
            self.size = size
        if red is not None:
            self.red = red
        if green is not None:
            self.green = green
        if blue is not None:
            self.blue = blue
        if alpha is not None:
            self.alpha = alpha
        if shape is not None:
            self.shape = shape
        if gradient is not None:
            self.gradient = gradient
        if rate is not None:
            self.rate = rate


from setuptools import setup, find_packages

setup(
    name='particle_systemszcz',
    version='1.3.2',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    install_requires=[
        'pygame-ce',
        'pygame-gui'
    ],
    author='rutra',
    author_email='ruterszcz@gmail.com',
    description='A particle system library',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/rutra8002/particle_system',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)